#!/bin/sh
#########################################################################
#
# RUNFILE FOR PINOCCHIO - V. 1.0
#
#########################################################################

#
# Here all the variables are defined
#

kernel=lcdm00              # kernel name of the run *** MUST BE 6 CHAR LONG ***
grid=128                   # grid number
omega0=0.3                 # Omega_0
omlam=0.7                  # Omega_Lambda at z=0 (NB if nonzero then Omega_0 + Omega_Lambda = 1)
nsp=-10.                   # Spectral parameter (see documentation)
sigma8=0.9                 # sigma_8
h=0.65                     # Hubble constant
lbox=200.                  # Box size (Mpc or Mpc/h)
flag=0                     # =0 if lbox is in Mpc, =1 if lbox is in Mpc/h
seed=3337322               # seed for random number generator
mode_flag=0                # 0 for random modules, 1 to fix them to average
nout=4                     # number of intermediate outputs
zfin=0.00                  # final redshift
z1=4.00                    # 
z2=3.00                    # intermediate redshifts
z3=2.00                    #
z4=1.00                    #

# NB the tmp directory is created within the rundir directory.
#    For 256^3 it is going to require about 2 Gb.  If you want it
#    in some other disk, you should create it and link it to the 
#    $kernel directory.
#    This means that you have to comment out the line below
#
#    mkdir $kernel/tmp/
#
#    and after the line
#
#    cd $kernel
#
#    add the line
# 
#    link -s [my tmp directory] tmp

#
# Here it creates the file fragment.info
#

echo "$kernel         ! run                              " >  fragment.info
echo "$kernel         ! kernel                           " >> fragment.info
echo "$zfin           ! z fin                            " >> fragment.info
echo "$nout           ! # outputs                        " >> fragment.info
echo "$z1 $z2 $z3 $z4 ! z out                            " >> fragment.info
echo "0               ! parameter flag (0=default)       " >> fragment.info
echo "4               ! out_flag                         " >> fragment.info
echo "1               ! stop_flag  (0=run again, 1=stop) " >> fragment.info
echo "0               ! limit flag (0=no limits)         " >> fragment.info

########################################################
# You might have no need to edit the rest of this file #
#    (unless you want to change the tmp directory)     #
########################################################


#
# Step 1: creates the directory and the .info file
#

mkdir $kernel              2> error.log 
mkdir $kernel/Data/        2>> error.log 
mkdir $kernel/tmp/         2>> error.log 
mv fragment.info $kernel/  2>> error.log 
cd $kernel
ln -s ../make_run.x .      2>> ../error.log 
ln -s ../mf.x .            2>> ../error.log 
ln -s ../init.x .          2>> ../error.log 
ln -s ../fmax_ell.x .      2>> ../error.log 
ln -s ../fragment.x .      2>> ../error.log 
cp ../mf.sm .              2>> ../error.log 
touch core; chmod 000 core 2>> ../error.log 

./make_run.x  > make_run.log 2>&1 <<EOF
$kernel
$flag
$lbox
$omega0
$omlam
$nsp
$sigma8       
$h
$grid
EOF

#
# Step 2: creates the initial density field
#

./init.x > init.log 2>&1 <<EOF
$seed
$kernel
$mode_flag
EOF

#
# Step 3: creates Fmax, Rmax and Vmax
#

./fmax_ell.x > fmax.log 2>&1 <<EOF
$kernel
EOF

\rm -f tmp/*

#
# Step 4: fragments the collapsed medium
#

./fragment.x > fragment.log 2>&1

#
# Step 5: analytic mass functions
#

./mf.x $omega0 $omlam $nsp $sigma8 $h $zfin 0 mf.ps.$zfin.out > mf.log   2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $zfin 1 mf.st.$zfin.out >> mf.log  2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $zfin 2 mf.jn.$zfin.out >> mf.log  2>&1 
								
./mf.x $omega0 $omlam $nsp $sigma8 $h $z1   0 mf.ps.$z1.out   >> mf.log  2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $z1   1 mf.st.$z1.out   >> mf.log  2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $z1   2 mf.jn.$z1.out   >> mf.log  2>&1 
					    		      	
./mf.x $omega0 $omlam $nsp $sigma8 $h $z2   0 mf.ps.$z2.out   >> mf.log  2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $z2   1 mf.st.$z2.out   >> mf.log  2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $z2   2 mf.jn.$z2.out   >> mf.log  2>&1 
					    		      	
./mf.x $omega0 $omlam $nsp $sigma8 $h $z3   0 mf.ps.$z3.out   >> mf.log  2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $z3   1 mf.st.$z3.out   >> mf.log  2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $z3   2 mf.jn.$z3.out   >> mf.log  2>&1 
					    		      	
./mf.x $omega0 $omlam $nsp $sigma8 $h $z4   0 mf.ps.$z4.out   >> mf.log  2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $z4   1 mf.st.$z4.out   >> mf.log  2>&1 
./mf.x $omega0 $omlam $nsp $sigma8 $h $z4   2 mf.jn.$z4.out   >> mf.log  2>&1 

cd ../

