c Package: PINOCCHIO 1.1
c
c Written by Pierluigi Monaco
c Copyright (C) 2001
c
      program Fmax_ell
c
c Predicts the regions in OC from the initial conditions
c according to ellipsoidal collapse
c Gives 3rd order velocities at Rmax
c
      implicit none
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      real*8 fk2(ng2,ng,ng),fkc(ng),lbox8
      real lbox
      common /cmodes/lbox8,fk2,fkc
      character run*6,tmp_dir*4
      integer start_flag
      common /start/start_flag
c
      print *,'This version is compiled with a resolution of ',ng,' cube'
      start_flag=0
      print *,'run?'
      read '(a)',run
      tmp_dir='tmp/'
c
c Smoothing radii
c
      call radii(lbox,run)
      lbox8=lbox
c
c Modes
c
      call modes_()
      print *,'Modes generated'
      call flush(6)
c
c Reads the initial conditions, FFT's and saves them in a temp file
c
      call read_init(run,tmp_dir)
c
c Predicts the OC regions
c
      call oc_predicter(tmp_dir,run)
c
c Writes the R_c curves
c
      call write_Fmax(tmp_dir,run)
c
c bye!
c
      stop
      end
c
c
c
      subroutine read_init(run,tmp_dir)
c
c Reads the initial conditions, performs FFTs and saves the results
c
c
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      real d(ng,ng,ng),dn(2,ng,ng)
      character*80 nome,run*6,tmp_dir*4
c
      nome='Data/'//run//'.dl'
      open(10,file=nome,status='old',form='unformatted')
      read(10)d
      close(10)
      print *,'Initial conditions read'
      call flush(6)
c
      do k=1,ng
         do j=1,ng
            dn(1,j,k)=0.
            dn(2,j,k)=0.
         enddo
      enddo
      call fft(d,dn,ng,1)
      print *,'FFT done'
      call flush(6)
c
      nome=tmp_dir//'temp_d.out'
      open(10,file=nome,status='unknown',form='unformatted')
      write(10)d,dn
      close(10)
      print *,'FFT saved on file '//nome
      call flush(6)
c
      return
      end
c
c
c
      subroutine oc_predicter(tmp_dir,run)
c
c Predicts the regions in OC, for several smoothing radii,
c and constructs the R_c(q) curve.
c
      implicit none
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
c
      real d(ng,ng,ng),dn(2,ng,ng)
      real*8 d8(ng,ng,ng),dn8(2,ng,ng)
      real s0
      integer nstep
c
      real*8 fk2(ng2,ng,ng),fkc(ng),lbox
      common /cmodes/lbox,fk2,fkc
      real rsm(nrsmax),var(nrsmax),var_true(nrsmax)
      common /smoothing/rsm,var,nstep
      integer filter,istep,i,j,k
      common /smoothing_filter/filter
      common /field/d8,dn8
      integer start_flag
      common /start/start_flag
c
      character*80 nome,tmp_dir*4,run*6
c
c Cycle on smoothing scales
c
      do istep=1,nstep
         print *
         print *,'Smoothing: ',istep,rsm(istep),sqrt(var(istep))
         call flush(6)
c
c Reads the unsmoothed FFT density field
c
         nome=tmp_dir//'temp_d.out'
         open(10,file=nome,status='old',form='unformatted')
         read(10)d,dn
         close(10)
         print *,'Initial conditions read from file '//nome
         call flush(6)
         do k=1,ng
            do j=1,ng
               do i=1,ng
                  d8(i,j,k)=d(i,j,k)
               enddo
               dn8(1,j,k)=dn(1,j,k)
               dn8(2,j,k)=dn(2,j,k)
            enddo
         enddo
c
c Smoothing
c
         if (rsm(istep).gt.0.) then
            call smootf_(rsm(istep))
            print *,'Smoothing done'
         endif
         call variance_fft(s0)
         print *,'RMS of the smoothed field: ',s0
         var_true(istep)=s0
         call flush(6)
c
c Poisson eqn
c
         call poisson_dest_()
         print *,'Poisson equation done'
         call flush(6)
c
c First derivative of grav. potential (swapped on files)
c
         call differ1_swap(tmp_dir)
         print *,'First derivative done'
         call flush(6)
c
c Second derivative of grav. potential (swapped on files)
c
         call differ2_swap(tmp_dir)
         print *,'Second derivative done'
         call flush(6)
c
c Ellipsoidal collapse
c
         call make_ell(istep,tmp_dir)
         print *,'Ellipsoidal collapse done'
         call flush(6)
c
c End of smoothing cycle
c
         if (start_flag.eq.0) start_flag=1
      enddo
c
c Print files with true variances
c
      open(10,file=run//'.var',status='unknown')
      do istep=1,nstep
         write(10,39)rsm(istep),sqrt(var(istep)),var_true(istep)
      enddo
      close(10)
      print *,'Written file '//run//'.var'
 39   format(1x,3(f12.6))
c
c Bye!
c
      print *,'OC_PREDICTER done'
      call flush(6)
      return
      end
c
c
c
      subroutine write_Fmax(tmp_dir,run)
c
c Writes the actual R_c curves
c
      implicit none
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      integer ng8,nsplit
      parameter(nsplit=32,ng8=ng/nsplit) 
      integer nstep,i,j,k,isplit,k1,ifile
      real rsm(nrsmax),var(nrsmax)
      common /smoothing/rsm,var,nstep
      real help(ng,ng,ng8),data(ng,ng,ng)
      character nome*80,label*2,tmp_dir*4,run*6
      character*4 lab_file(5)
      data lab_file/'Fmax','Rmax','vmxx','vmxy','vmxz'/
c
c Writes the results
c
      do ifile=1,5
c
         do isplit=1,nsplit
            write(label,'(i2)')isplit
            if (label(1:1).eq.' ') label(1:1)='0'
            nome=tmp_dir//lab_file(ifile)//'_'//label//'.tmp'
            open (20,file=nome,status='unknown',form='unformatted')
            read (20)help
            close(20)
c            print *,'Read file '//nome
            do k=1,ng8
               k1=(isplit-1)*ng8+k
               do j=1,ng
                  do i=1,ng
                     data(i,j,k1)=help(i,j,k)
                  enddo
               enddo
            enddo
         enddo
         nome='Data/'//lab_file(ifile)//'.ell.'//run//'.out'
         open(10,file=nome,status='unknown',form='unformatted')
         write(10)data
         close(10)
         print *,'Written file '//nome
         call flush(6)
c
      enddo
c
c Bye!
c
      return
      end
c
c
c
      subroutine differ1_swap(tmp_dir)
c
c First derivative of the potential
c The results are written on temp files
c
      implicit none
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      integer ng8,nsplit
      parameter(nsplit=32,ng8=ng/nsplit)
      real*8 d(ng,ng,ng),dn(2,ng,ng)
      common /field/d,dn
      real*8 fk2(ng2,ng,ng),fkc(ng),lbox
      common /cmodes/lbox,fk2,fkc
      real*8 swap(ng,ng,ng),swan(2,ng,ng)
      common /field8/swap,swan
      integer i,j,k,ia,iv(3),k1,isplit
      real*8 temp(ng,ng,ng8),f1,vol
      character nome*80,label*1,tmp_dir*4
c
c First derivatives of (d,dn)
c
      vol=2./float(ngtot)
      do ia=1,3
         do k=1,ng
            do j=1,ng
               do i=1,ng2
                  iv(1)=i
                  iv(2)=j
                  iv(3)=k
                  f1=fkc(iv(ia))
                  if (i.ne.ng2) then
                     swap(2*i-1,j,k) =  f1*d(2*i,j,k)
                     swap(2*i,j,k)   = -f1*d(2*i-1,j,k)
                  else
                     swan(1,j,k)     =  f1*dn(2,j,k)
                     swan(2,j,k)     = -f1*dn(1,j,k)
                  endif
               enddo
            enddo
         enddo
c
         call fft8(swap,swan,ng,-1)
         print *,'DIFFER1, FFT number',ia,' done'
         call flush(6)
c
         do i=1,ng
            do j=1,ng
               do k=1,ng
                  swap(i,j,k)=swap(i,j,k)*vol
               enddo
            enddo
         enddo
c
         write(label,'(i1)')ia
         nome=tmp_dir//'temp_d1_'//label//'.out'
         open(10,file=nome,status='unknown',form='unformatted')
         do isplit=1,nsplit
            do k=1,ng8
               k1=(isplit-1)*ng8+k
               do j=1,ng
                  do i=1,ng
                     temp(i,j,k)=swap(i,j,k1)
                  enddo
               enddo
            enddo
            write(10)temp
         enddo
         close(10)
         print *,'Written file '//nome
         call flush(6)
c
      enddo
c
c Bye!
c
      return
      end
c
c
c
      subroutine differ2_swap(tmp_dir)
c
c Second derivative of the potential
c The results are written on temp files
c
      implicit none
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      integer ng8,nsplit
      parameter(nsplit=32,ng8=ng/nsplit)
      real*8 d(ng,ng,ng),dn(2,ng,ng)
      common /field/d,dn
      real*8 fk2(ng2,ng,ng),fkc(ng),lbox
      common /cmodes/lbox,fk2,fkc
      real*8 swap(ng,ng,ng),swan(2,ng,ng)
      common /field8/swap,swan
      integer i,j,k,ia,ib,iv(3),k1,m,isplit
      real*8 temp(ng,ng,ng8)
      real*8 f1,f2,vol
      character nome*80,label*1,tmp_dir*4
c
c Second derivatives of (d,dn)
c
      vol=2./float(ngtot)
      do ia=1,3
         do ib=ia,3
            do j=1,ng
               do k=1,ng
                  do i=1,ng2
                     iv(1)=i
                     iv(2)=j
                     iv(3)=k
                     f1=fkc(iv(ia))
                     f2=fkc(iv(ib))
                     if (i.ne.ng2) then
                        swap(2*i-1,j,k)=-f1*f2*d(2*i-1,j,k)
                        swap(2*i,j,k)=-f1*f2*d(2*i,j,k)
                     else
                        swan(1,j,k)=-f1*f2*dn(1,j,k)
                        swan(2,j,k)=-f1*f2*dn(2,j,k)
                     endif
                  enddo
               enddo
            enddo
c
            if (ia.eq.ib) then
               m=ia
            else
               m=ia+ib+1
            endif
            call fft8(swap,swan,ng,-1)
            print *,'DIFFER2, FFT number',m,' done'
            call flush(6)
c
            do i=1,ng
               do j=1,ng
                  do k=1,ng
                     swap(i,j,k)=swap(i,j,k)*vol
                  enddo
               enddo
            enddo
c
            write(label,'(i1)')m
            nome=tmp_dir//'temp_d2_'//label//'.out'
            open(10,file=nome,status='unknown',form='unformatted')
            do isplit=1,nsplit
               do k=1,ng8
                  k1=(isplit-1)*ng8+k
                  do j=1,ng
                     do i=1,ng
                        temp(i,j,k)=swap(i,j,k1)
                     enddo
                  enddo
               enddo
               write(10)temp
            enddo
            close(10)
            print *,'Written file '//nome
            call flush(6)
c
         enddo
      enddo
c
c Bye!
c
      return
      end
c
c
c
      subroutine make_ell(istep,tmp_dir)
c
c Second derivative of the potential
c The results are written on temp files
c
      implicit none
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      integer ng8,nsplit
      parameter(nsplit=32,ng8=ng/nsplit)
      real*8 d1(3,ng,ng,ng8),d2(6,ng,ng,ng8),temp(ng,ng,ng8)
      real Fmax(ng,ng,ng8),Rmax(ng,ng,ng8)
      real vmax_x(ng,ng,ng8),vmax_y(ng,ng,ng8),vmax_z(ng,ng,ng8)
      integer i,j,k,ia,m,isplit,nfail,nc
      real*8 mu1,mu2,mu3,c,q,r,sq,t
      real*8 x1,x2,x3,dpi,v(3)
      character*80 nome,label*2
      real rsm(nrsmax),var(nrsmax),f
      common /smoothing/rsm,var,nstep
      integer istep,nstep
      real*8 ell_
      external ell_
      character*4 tmp_dir
      integer start_flag
      common /start/start_flag
c
c Eigenvalues
c
      nome=tmp_dir//'temp_d2_1.out'
      open(11,file=nome,status='old',form='unformatted')
      nome=tmp_dir//'temp_d2_2.out'
      open(12,file=nome,status='old',form='unformatted')
      nome=tmp_dir//'temp_d2_3.out'
      open(13,file=nome,status='old',form='unformatted')
      nome=tmp_dir//'temp_d2_4.out'
      open(14,file=nome,status='old',form='unformatted')
      nome=tmp_dir//'temp_d2_5.out'
      open(15,file=nome,status='old',form='unformatted')
      nome=tmp_dir//'temp_d2_6.out'
      open(16,file=nome,status='old',form='unformatted')
      nome=tmp_dir//'temp_d1_1.out'
      open(17,file=nome,status='old',form='unformatted')
      nome=tmp_dir//'temp_d1_2.out'
      open(18,file=nome,status='old',form='unformatted')
      nome=tmp_dir//'temp_d1_3.out'
      open(19,file=nome,status='old',form='unformatted')
c
      nfail=0
      nc=0
      do isplit=1,nsplit
c
c Second set of temp files
c
         write(label,'(i2)')isplit
         if (label(1:1).eq.' ') label(1:1)='0'
         if (start_flag.eq.0) then
c
c If first step, then init
c
            do k=1,ng8
               do j=1,ng
                  do i=1,ng
                     Fmax(i,j,k)=-1.e9
                     Rmax(i,j,k)=0.
                     vmax_x(i,j,k)=0.
                     vmax_y(i,j,k)=0.
                     vmax_z(i,j,k)=0.
                  enddo
               enddo
            enddo
         else
c
            nome=tmp_dir//'Fmax_'//label//'.tmp'
            open (20,file=nome,status='unknown',form='unformatted')
            read (20)Fmax
            close(20)
c
            nome=tmp_dir//'Rmax_'//label//'.tmp'
            open (20,file=nome,status='unknown',form='unformatted')
            read (20)Rmax
            close(20)
c
            nome=tmp_dir//'vmxx_'//label//'.tmp'
            open (20,file=nome,status='unknown',form='unformatted')
            read (20)vmax_x
            close(20)
c
            nome=tmp_dir//'vmxy_'//label//'.tmp'
            open (20,file=nome,status='unknown',form='unformatted')
            read (20)vmax_y
            close(20)
c
            nome=tmp_dir//'vmxz_'//label//'.tmp'
            open (20,file=nome,status='unknown',form='unformatted')
            read (20)vmax_z
            close(20)
c
c            print *,'temp files read'
         endif
c
c Reads a segment of the temp files
c
c         print *,isplit
         call flush(6)
c first derivatives
         do m=1,3
            read(16+m,end=10)temp
c            print *,'Reading temp file ',m
            call flush(6)
            do k=1,ng8
               do j=1,ng
                  do i=1,ng
                     d1(m,i,j,k)=temp(i,j,k)
                  enddo
               enddo
            enddo
         enddo
c second derivatives
         do m=1,6
            read(10+m,end=10)temp
c            print *,'Reading temp file ',m+3
            call flush(6)
            do k=1,ng8
               do j=1,ng
                  do i=1,ng
                     d2(m,i,j,k)=temp(i,j,k)
                  enddo
               enddo
            enddo
         enddo
c
c Calculates the ellipsoidal collapse
c
         do k=1,ng8
            do j=1,ng
               do i=1,ng
                  mu1=d2(1,i,j,k)+d2(2,i,j,k)+d2(3,i,j,k)
                  mu2=.5*mu1*mu1
                  do m=1,6
                     if (m.le.3) then 
                        c=.5
                     else
                        c=1.
                     endif
                     mu2=mu2-c*d2(m,i,j,k)*d2(m,i,j,k)
                  enddo
                  mu3=d2(1,i,j,k)*d2(2,i,j,k)*d2(3,i,j,k)
     $              + 2.*d2(4,i,j,k)*d2(5,i,j,k)*d2(6,i,j,k)
     $                 - d2(1,i,j,k)*d2(6,i,j,k)*d2(6,i,j,k)
     $                 - d2(2,i,j,k)*d2(5,i,j,k)*d2(5,i,j,k)
     $                 - d2(3,i,j,k)*d2(4,i,j,k)*d2(4,i,j,k)
                  q=(mu1*mu1-3.*mu2)/9.
                  r=-(2.*mu1**3-9.*mu1*mu2+27.*mu3)/54.
c ***
                  if (q*q*q.lt.r*r.or.q.lt.0.) then
                     nfail=nfail+1
                     goto 7
                  endif
c ***
                  sq=sqrt(q)
                  t=acos(r/q/sq)
                  x1=-2.*sq*cos(t/3.)+mu1/3.
                  x2=-2.*sq*cos((t+2.*pi)/3.)+mu1/3.
                  x3=-2.*sq*cos((t+4.*pi)/3.)+mu1/3.
                  call ord_(x1,x2,x3)
                  dpi=pi
                  t=ell_(x1,x2,x3,mu1,mu3,dpi)
c
c inverse collapse time
c
                  if (t.ne.0.) then 
                     f=1./t
                  else
                     f=-1.e8
                  endif
c
c Velocities
c
                  do ia=1,3
                     v(ia)=-d1(ia,i,j,k)
                  enddo
c
c Updates files
c
                  if (f.gt.Fmax(i,j,k)) then
                     Fmax(i,j,k)=f
                     Rmax(i,j,k)=rsm(istep)
                     vmax_x(i,j,k)=v(1)
                     vmax_y(i,j,k)=v(2)
                     vmax_z(i,j,k)=v(3)
                     nc=nc+1
                  endif
 7             enddo
            enddo
         enddo
c
c Closes second set of files
c
         nome=tmp_dir//'Fmax_'//label//'.tmp'
         open (20,file=nome,status='unknown',form='unformatted')
         write(20)Fmax
         close(20)
c
         nome=tmp_dir//'Rmax_'//label//'.tmp'
         open (20,file=nome,status='unknown',form='unformatted')
         write(20)Rmax
         close(20)
c
         nome=tmp_dir//'vmxx_'//label//'.tmp'
         open (20,file=nome,status='unknown',form='unformatted')
         write(20)vmax_x
         close(20)
c
         nome=tmp_dir//'vmxy_'//label//'.tmp'
         open (20,file=nome,status='unknown',form='unformatted')
         write(20)vmax_y
         close(20)
c
         nome=tmp_dir//'vmxz_'//label//'.tmp'
         open (20,file=nome,status='unknown',form='unformatted')
         write(20)vmax_z
         close(20)
c
c End of splitting cycle
c
      enddo
      print *,'Number of failed attempts in diagonalization: ',nfail
      print *,'Ell fmax curve updated: ',nc
      do m=11,19
         close(m)
      enddo
      call flush(6)
      return
 10   print *,'EOF in temp file number ',m
      do m=11,19
         close(m)
      enddo
      call flush(6)
      return
      end
c
c
c
      function ell_(l1,l2,l3,del,det,pi)
c
c It computes the smallest non-negative solution of the 3rd order 
c equation for the ellipsoid, and corrects it to reproduce the
c spherical collapse correctly.
c
      real*8 ell_,a1,a2,a3,del,det,den,dis,r,q,sq,t,s1,s2,s3,pi,l1,l2,l3
      den=det/126.+5.*l1*del*(del-l1)/84.
      if (l1.eq.0.) then
         ell_=-.1
      else if (den.eq.0.) then
         if (del.eq.l1) then
            if (l1.gt.0) then
               ell_=1./l1
            else
               ell_=-.1
            endif
         else
            dis=7.*l1*(l1+6.*del)
            if (dis.lt.0.) then
               ell_=-.1
            else
               ell_=(7.*l1-sqrt(dis))/(3.*l1*(l1-del))
               if (ell_.lt.0.) ell_=-.1
            endif
         endif
      else
         a1=3.*l1*(del-l1)/14./den
         a2=l1/den
         a3=-1./den
         q=(a1*a1-3.*a2)/9.
         r=(2.*a1*a1*a1-9.*a1*a2+27.*a3)/54.
         if (r*r.gt.q*q*q) then
            sq=(sqrt(r*r-q*q*q)+abs(r))**(1./3.)
            ell_=-abs(r)/r*(sq+q/sq)-a1/3.
            if (ell_.lt.0.) ell_=-.1
         else
            sq=sqrt(q)
            t=acos(r/q/sq)
            s1=-2.*sq*cos(t/3.)-a1/3.
            s2=-2.*sq*cos((t+2.*pi)/3.)-a1/3.
            s3=-2.*sq*cos((t+4.*pi)/3.)-a1/3.
            if (s1.lt.0.) s1=1.d10
            if (s2.lt.0.) s2=1.d10
            if (s3.lt.0.) s3=1.d10
            ell_=min(s1,s2,s3)
            if (ell_.eq.1.d10) ell_=-.1
         endif
      endif
      if (del.gt.0..and.ell_.gt.0.) then
         corr=-.364/del*exp(-6.5*(l1-l2)/del-2.8*(l2-l3)/del)
      else
         corr=0.
      endif
      ell_=ell_+corr
      return
      end
c
c
c
      subroutine ord_(a,b,c)
c
c Ordina tre numeri
c
      real*8 a,b,c,a1,b1,c1
      a1=max(a,b,c)
      c1=min(a,b,c)
      b1=a+b+c-a1-c1
      a=a1
      b=b1
      c=c1
      return
      end
c
c
c
      subroutine poisson_dest_()
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      real*8 pot(ng,ng,ng),potn(2,ng,ng)
      common /field/pot,potn
      real*8 fk2(ng2,ng,ng),fkc(ng),lbox
      common /cmodes/lbox,fk2,fkc
c
      do k=1,ng
         do j=1,ng
            do i=1,ng2
               if (fk2(i,j,k).eq.0.) then
                  pot(2*i-1,j,k)=0.
                  pot(2*i,j,k)=0.
               else
                  if (i.ne.ng2) then
                     pot(2*i-1,j,k)=-pot(2*i-1,j,k)/fk2(i,j,k)
                     pot(2*i,j,k)=-pot(2*i,j,k)/fk2(i,j,k)
                  else
                     potn(1,j,k)=-potn(1,j,k)/fk2(i,j,k)
                     potn(2,j,k)=-potn(2,j,k)/fk2(i,j,k)
                  endif
               endif
            enddo
         enddo
      enddo
      return
      end
c
c
c
      subroutine smootf_(rsm)
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      real*8 del(ng,ng,ng),deln(2,ng,ng)
      real*8 fk2(ng2,ng,ng),fkc(ng),lbox
      real fil,rsm
      integer filter
      common /smoothing_filter/filter
      common /field/del,deln
      common /cmodes/lbox,fk2,fkc
      do k=1,ng
         do j=1,ng
            do i=1,ng2
               rr=sqrt(fk2(i,j,k))*rsm
               fil=window(rr)
               if (i.ne.ng2) then
                  del(2*i-1,j,k)=del(2*i-1,j,k)*fil
                  del(2*i,j,k)=del(2*i,j,k)*fil
               else
                  deln(1,j,k)=deln(1,j,k)*fil
                  deln(2,j,k)=deln(2,j,k)*fil
               endif
            enddo
         enddo
      enddo
      return
      end
c
c
c
      subroutine modes_()
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      real*8 fk2(ng2,ng,ng),fkc(ng),lbox
      common /cmodes/lbox,fk2,fkc
      do i=1,ng
         if (i.le.ng2) then
            k=i-1
         else
            k=-ng-1+i
         endif
         fkc(i)=2.*pi*float(k)/lbox
      enddo
      do j=1,ng
         do k=1,ng
            do i=1,ng2
               fk2(i,j,k)=fkc(i)*fkc(i)+fkc(j)*fkc(j)+fkc(k)*fkc(k)
            enddo
         enddo
      enddo
      return
      end
c
c
c
      subroutine variance_fft(s0)
c
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      real*8 dt(ng,ng,ng),dn(2,ng,ng)
      common /field/dt,dn
      real s0,vol
      integer is
      external is
      vol=1./float(ng**3)
      s0=0.
      do l=1,ng
         if (l.le.ng2) then
            i=l
         else
            i=is(l,ng)
         endif
         do j=1,ng
            do k=1,ng
               if (i.ne.ng2) then
                  s0=s0+(dt(2*i-1,j,k)**2+dt(2*i,j,k)**2)
               else
                  s0=s0+(dn(1,j,k)**2+dn(2,j,k)**2)
               endif                  
            enddo
         enddo
      enddo
      s0=sqrt(s0)*vol
      return
      end
