c Package: PINOCCHIO 1.1
c
c Written by Pierluigi Monaco
c Copyright (C) 2001
c
      program init
c
c Produces a realization of the initial density for a given cosmology
c
      implicit none
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      real dl(ng,ng,ng),dln(2,ng,ng)
c
      real fk2(ng2,ng,ng),fkc(ng)
      real omega0,omlam,nsp,csp,s8,h
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer filter
      common /smoothing_filter/filter
      real lbox,volfft
      integer i,j,k,seed,mode_flag
      character*80 nome,run*6
c
c NB ALL DISTANCES IN MPC, WITHOUT H!!!
c
c Cosmology
c
      print *,'This version is compiled with a resolution of ',ng,' cube'
c
      print *,'Seed for random numbers?'
      read *,seed
      print *,'  seed       = ',seed
      seed=-seed
      print *,'run?'
      read '(a)',run
 7    print *,'modules randomly extracted (0) or fixed to average (1)?'
      read *,mode_flag
      include 'run_info.inc'
c
      filter=2
      call normalization()
c
c Other init
c
      print *,'  Mbox       = ',log10(lbox**3*2.778e11*h**2)
      print *,'  Mpart      = ',log10((lbox/float(ng))**3*2.778e11*h**2)
c
      volfft=2./float(ngtot)
c
c Start
c
      call modes(lbox,fk2,fkc)
      print *,'Modes done'
      call flush(6)
c
c Generates random CI
c
      call monte_carlo(dl,dln,seed,fk2,mode_flag)
      print *,'Random initial conditions generated'
      call flush(6)
c
c FFT
c
      call fft(dl,dln,ng,-1)
c
      do k=1,ng
         do j=1,ng
            do i=1,ng
               dl(i,j,k)=dl(i,j,k)*volfft
            enddo
         enddo
      enddo
      print *,'FFT done'
      call flush(6)
c
c Writes result
c
      nome='Data/'//run//'.dl'
      open (10,file=nome,status='unknown',form='unformatted')
      write(10)dl
      close(10)
      print *,'Written file '//nome
      call flush(6)
c
c Bye!
c
      stop
      end
c
c
c
      subroutine monte_carlo(dt,dtn,seed,fk2,flag)
c
c flag=0: modules are taken at random from the distribution
c flag=1: modules are set to their mean value
c
      implicit none
#ifdef v64
      include 'param_64.inc'
#endif
#ifdef v128
      include 'param_128.inc'
#endif
#ifdef v256
      include 'param_256.inc'
#endif
c      include 'param.inc'
      real dt(ng,ng,ng),dtn(2,ng,ng)
      real fk2(ng2,ng,ng)
      integer i,j,k,seed,is,flag,Ne
      parameter(Ne=16)
      real pk,random,power,fk,vol,lbox,fke
      real y,t
      external random,power,is
c
c builds the Gaussian fields in k-space
c
      lbox=2.*pi/sqrt(fk2(2,1,1))
      fke=0.8*pi/lbox*float(ng)
      vol=2.*pi*pi*(float(ng**2)/lbox)**3
      print *,'vol = ',vol
      do i=1,ng2
         do j=1,ng
            do k=1,ng
               fk=sqrt(fk2(i,j,k))
               pk=power(fk)*vol*exp(-(fk/fke)**Ne)
               if ((i.eq.1.or.i.eq.ng2).and.
     &             (j.eq.1.or.j.eq.ng2).and.
     &             (k.eq.1.or.k.eq.ng2)) then
                  if (flag.eq.0) then
                     y=sqrt(-pk*log(random(seed)))
                  else
                     y=sqrt(pk)
                  endif
                  if (i.ne.ng2) then
                     dt(2*i-1,j,k) = y
                     dt(2*i,j,k) = 0.d0
                  else
                     dtn(1,j,k) = y
                     dtn(2,j,k) = 0.d0
                  endif
               else
                  t=2.*pi*random(seed)
                  if (flag.eq.0) then
                     y=sqrt(-pk*log(random(seed)))
                  else
                     y=sqrt(pk)
                  endif
                  if (i.ne.ng2) then
                     dt(2*i-1,j,k) = y*cos(t)
                     dt(2*i,j,k) = y*sin(t)
                  else
                     dtn(1,j,k) = y*cos(t)
                     dtn(2,j,k) = y*sin(t)
                  endif
               endif
c symmetrical components
               if (i.eq.1) then
                  dt(1,is(j,ng),is(k,ng)) = dt(1,j,k)
                  dt(2,is(j,ng),is(k,ng)) =-dt(2,j,k)
               else if (i.eq.ng2) then
                  dtn(1,is(j,ng),is(k,ng)) = dtn(1,j,k)
                  dtn(2,is(j,ng),is(k,ng)) =-dtn(2,j,k)
               endif
            enddo
         enddo
      enddo
      dt(1,1,1)=0.
      dt(2,1,1)=0.
c
      return
      end
