c Package: PINOCCHIO 1.1
c
c Written by Pierluigi Monaco
c Copyright (C) 2001
c
      program mass_function
c
c Produces the analytical mass function according to 
c Press & Schechter (1974), Sheth & Tormen (1999) or
c Jenkins et al. (2001)
c
      implicit none
      real omega0,omlam,nsp,csp,s8,h
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer filter,flag_ps
      common /smoothing_filter/filter
      real delta_c
      common /ps/delta_c,flag_ps
      external scale,time
      real z,mf,m,logm,an_h,safelog,aM_star
      character*80 string,nome
      external an_h,safelog,aM_star
      integer iargc
      external iargc
c
c Reads arguments
c
      if (iargc().ne.8) then
         print *,'usage:  mf.x Omega_0 Omega_Lambda nsp sigma_8 h z flag output_file'
         print *
         print *,'Omega_0, Omega_Lambda, sigma_8 and h are referred to z=0'
         print *
         print *,'nsp:    -10          Bardeen et al. (1986) fit'
         print *,'        -(10+Gamma)  as above with Gamma as specified'
         print *,'        10           Efstathiou, Bond & White (1992) fit'
         print *,'        10+Gamma     as above with Gamma as specified'
         print *,'        -3<= nsp <=1 power law spectrum'
         print *
         print *,'flag:   0  Press & Schechter (1974)'
         print *,'        1  Sheth & Tormen    (1999)'
         print *,'        2  Jenkins et al.    (2001)'
         stop
      endif
c
      call getarg(1,string)
      read(string,*) omega0
      call getarg(2,string)
      read(string,*)omlam
      call getarg(3,string)
      read(string,*)nsp
      call getarg(4,string)
      read(string,*)s8
      call getarg(5,string)
      read(string,*)h
      call getarg(6,string)
      read(string,*)z
      call getarg(7,string)
      read(string,*)flag_ps
      call getarg(8,nome)
c
      print *,'Omega_0      = ',omega0
      print *,'Omega_Lambda = ',omlam
      print *,'nsp          = ',nsp
      print *,'sigma_8      = ',s8
      print *,'h            = ',h
      print *,'z            = ',z
      print *,'flag         = ',flag_ps
      print *,'output file  = '//nome
c      
      delta_c=1.67509765
      call normalization()
      filter=2
      call grow_start()
      call scale_start()
c
      call Lambda_start()
c
      open(10,file=nome,status='unknown')
      write(10,101)'# Omega_0      = ',omega0
      write(10,101)'# Omega_Lambda = ',omlam
      write(10,101)'# nsp          = ',nsp
      write(10,101)'# sigma_8      = ',s8
      write(10,101)'# h            = ',h
      write(10,101)'# z            = ',z
      write(10,102)'# flag         = ',flag_ps
      write(10,103)'# M* a z=',z,' : ',aM_star(z)
      do logm=4.,18.,.1
         m=10.**logm
         mf=an_h(m,z)
         write(10,100)logm,safelog(mf)
      enddo
      print *,'M* a z=',z,': ',aM_star(z)
      close(10)
 100  format(50(1x,f12.6))
 101  format(a17,f12.6)
 102  format(a17,i2)
 103  format(a9,f5.2,a3,e12.4)
c
c Bye!
c
      stop
      end
      
