c Package: PINOCCHIO 1.1
c 
c Routines written by Pierluigi Monaco
c   adapted from Numerical Recipes in Fortran,
c   W.H. Press, S.A. Teukolsky, W.T. Vettering, B.P. Flannery,
c   1992, Cambridge University Press 
c
      function random(seed)
c
c Random number generator
c
      implicit none
      integer seed
      real random
c parameters
      integer m1,a1,q1,r1,m2,a2,q2,r2,m3
      integer div,med
      real small,rea_1,rea_2
      parameter (m1=2147483563
     $          ,a1=40014
     $          ,q1=53668
     $          ,r1=12211
     $          ,m2=2147483399
     $          ,a2=40692
     $          ,q2=52774
     $          ,r2=3791
     $          ,m3=m1-1
     $          ,div=32
     $          ,med=1+m3/div)
      parameter (small=1.2e-7
     $          ,rea_1=1./m1
     $          ,rea_2=1.-small)
c
      integer seed_save,i,j,k,iv(div)
      save seed_save,k,iv
      data seed_save/327104/
      data iv/div*0/
      data k/0/
c
      if (seed.eq.0) stop 'SEED = 0 in RANDOM'
      if (seed.lt.0) then
c initialization
         seed=-seed
         seed_save=seed
         do i=div+8,1,-1
            j=seed/q1
            seed=a1*(seed-j*q1)-j*r1
            if (seed.lt.0) then
               seed=seed+m1
            endif
            if (i.le.div) iv(i)=seed
         enddo
         k=iv(1)
      endif
c updates seed
      j=seed/q1
      seed=a1*(seed-j*q1)-j*r1
      if (seed.lt.0) then
         seed=seed+m1
      endif
c updates saved seed
      j=seed_save/q2
      seed_save=a2*(seed_save-j*q2)-j*r2
      if (seed_save.lt.0) then
         seed_save=seed_save+m2
      endif
      i=1+k/med
      k=iv(i)-seed_save
      iv(i)=seed
      if (k.lt.1) k=k+m3
c calculates random number
      random=min(rea_1*k,rea_2)
      return
      end
c
c
c
      subroutine index_sort(n,vector,index)
c
c performs indexed sorting of a vector
c using the quicksort algorithm
c
      integer n,index(n)
      real vector(n)
c
      integer m,nsave
      parameter (m=7,nsave=100)
c
      integer save(nsave),isave
      integer i,ii,j,k,l,indx,tmp
      real c
c init
      do i=1,n
        index(i)=i
      enddo
      isave=0
      k=1
      ii=n
c main loop
      do while (.true.)
         if (ii-k.lt.m) then
c straight insertion
            do j=k+1,ii
               indx=index(j)
               c=vector(indx)
               do i=j-1,1,-1
                  if (vector(index(i)).le.c) goto 11
                  index(i+1)=index(i)
               enddo
               i=0
 11            index(i+1)=indx
            enddo
c finished?
            if (isave.eq.0) return
c update
            ii=save(isave)
            k=save(isave-1)
            isave=isave-2
c
         else
c partition exchange
            l=(k+ii)/2
            tmp=index(l)
            index(l)=index(k+1)
            index(k+1)=tmp
c
            if (vector(index(k+1)).gt.vector(index(ii))) then
               tmp=index(k+1)
               index(k+1)=index(ii)
               index(ii)=tmp
            endif
c
            if (vector(index(k)).gt.vector(index(ii))) then
               tmp=index(k)
               index(k)=index(ii)
               index(ii)=tmp
            endif
c
            if (vector(index(k+1)).gt.vector(index(k))) then
               tmp=index(k+1)
               index(k+1)=index(k)
               index(k)=tmp
            endif
c
            i=k+2
            j=ii-1
            indx=index(k)
            c=vector(indx)
c
            do while (.true.)
               do while (vector(index(i)).lt.c)
                  i=i+1
               enddo
               do while (vector(index(j)).gt.c) 
                  j=j-1
               enddo
               if (j.lt.i) goto 12
               tmp=index(i)
               index(i)=index(j)
               index(j)=tmp
               i=i+1
               j=j-1
c
            enddo
c
 12         index(k)=index(j)
            index(j)=indx
            isave=isave+2
c
            if (isave.gt.nsave) stop 'Increase nsave'
c
            if (ii-i+1.ge.j-k) then
               save(isave)=ii
               save(isave-1)=i
               ii=j-1
            else
               save(isave)=j-1
               save(isave-1)=k
               k=i
            endif
c
         endif
      enddo
      end
c
c
c
      subroutine integrate(f,x1,x2,result)
c
c Romberg's integration routine
c
      implicit none
      real f,x1,x2,result
      external f
c
      integer nmax,nextr
      parameter (nmax=15,nextr=5)
      real small
      parameter (small=1.e-6)
      integer i
      real p(nmax+1),r(nmax+1),err
c
      p(1)=1.
      r(1)=0.
      call trapezoidal (1,f,x1,x2,r(1))
c
      do i=2,nmax
         p(i)=0.25*p(i-1)
         r(i)=r(i-1)
         call trapezoidal(i,f,x1,x2,r(i))
c
         if (i.ge.nextr) then
            call polinomial(p(i-nextr+1),r(i-nextr+1)
     $           ,nextr,0.,result,err)
            if (abs(err).le.abs(result)*small) return
         endif
c
      enddo
      print *,'Warning: integration did not converge ',result,err
      return
c
      end
c
c
c
      subroutine trapezoidal(n,f,x1,x2,result)
c
c Integral evaluation with trapezoidal rule of order n
c
      implicit none
      integer n
      real f,x1,x2,result
      external f
c
      integer ng,i
      real deltax,dx,x,step
c
      deltax=x2-x1
      if (n.eq.1) then
         result=deltax*0.5*(f(x1)+f(x2))
      else
         ng=2**(n-2)
         dx=deltax/float(ng)
         step=0.
         do i=1,ng
            x=x1+(-0.5+i)*dx
            step=step+f(x)
         enddo
         result=0.5*(result+dx*step)
      endif
c
      return
      end
c
c
c
      subroutine polinomial(x,y,n,xp,yp,err)
c
c Polynomial interpolation with Neville's algorithm
c
      implicit none
      integer n,nmax
      parameter (nmax=20)
      real x(n),y(n),xp,yp,err
c
      integer i,m,nearest
      real C(nmax),D(nmax),a,b,e
c initializes C and D coefficients
      do i=1,n
         C(i)=y(i)
         D(i)=y(i)
      enddo
c finds nearest grid point
      a=100.*abs(x(n)-x(1))
      do i=1,n
         if (abs(xp-x(i)).lt.a) then
            a=abs(xp-x(i))
            nearest=i
         endif
      enddo
c first guess
      yp=y(nearest)
c iteration
      do m=1,n-1
         do i=1,n-m
            e=(C(i+1)-D(i))/(x(i)-x(i+m))
            a=(x(i)  -xp)*e
            b=(x(i+m)-xp)*e
            C(i)=a
            D(i)=b
         enddo
c further guess
         if (2*(nearest-1).lt.n-m) then
            err=C(nearest)
         else
            nearest=nearest-1
            err=D(nearest)
         endif
         yp=yp+err
c
      enddo
c         
      return
      end
c
c
c
      subroutine fft(field,fnyq,ng,dir)
c
c FFT of a 3D field for a cubic grid
c NB can be easily generalized to non-cubic grids!
c
      implicit none
      integer ng,dir,i,j,k
      real field(ng,ng,ng),fnyq(2,ng,ng)
c
      if (dir.eq.1) then
         call true_fft(ng,dir,field)
         do k=1,ng
            do j=1,ng
               do i=1,2
                  fnyq(i,j,k)=field(i,j,k)
               enddo
            enddo
         enddo
         call disentangle(ng,dir,field,fnyq)
      else
         call disentangle(ng,dir,field,fnyq)
         call true_fft(ng,dir,field)
      endif
c
c Bye!
c
      return
      end
c
c
c
      subroutine disentangle(ng,dir,field,fnyq)
c
c Disentangles the Nyquist frequencies
c
      implicit none
      integer ng,dir
      real field(ng,ng,ng),fnyq(2,ng,ng)
c
      real*8 pi8
      parameter (pi8=3.1415926535897932d0*2.d0)
      real*8 teta,real,imag,real_inc,imag_inc,temp
      integer i,j,k,ii,jj,kk
      real uno,due,tre,qua
c
      teta=pi8*dir/dble(ng)
      real_inc=-2.0d0*sin(0.5d0*teta)**2
      imag_inc=sin(teta)
c
c Disentangles Nyquist frequencies
c

      do k=1,ng
         kk=ng-k+2
         if (k.eq.1) kk=1 
c
         real=1.0d0
         imag=0.0d0
c
         do i=1,ng/4+1
            ii=ng/2-i+2
            do j=1,ng
               jj=ng-j+2
               if (j.eq.1) jj=1
c
               if (i.eq.1) then
                  uno=0.5*(field(1,j,k)+fnyq(1,jj,kk)
     $                 +dir*(field(2,j,k)+fnyq(2,jj,kk)))
                  due=0.5*(field(2,j,k)-fnyq(2,jj,kk)
     $                 +dir*(-field(1,j,k)+fnyq(1,jj,kk)))
                  tre=0.5*(field(1,j,k)+fnyq(1,jj,kk)
     $                 -dir*(field(2,j,k)+fnyq(2,jj,kk)))
                  qua=0.5*(-field(2,j,k)+fnyq(2,jj,kk)
     $                 +dir*(-field(1,j,k)+fnyq(1,jj,kk)))
                  field(1,j,k)   = uno
                  field(2,j,k)   = due
                  fnyq (1,jj,kk) = tre
                  fnyq (2,jj,kk) = qua
               else
                  uno=0.5*(field(2*i-1,j,k)+field(2*ii-1,jj,kk)
     $                 + dir*(real*(field(2*i,j,k)+field(2*ii,jj,kk))
     $                 - imag*(-field(2*i-1,j,k)+field(2*ii-1,jj,kk))))
                  due=0.5*(field(2*i,j,k)-field(2*ii,jj,kk)
     $                 + dir*(imag*(field(2*i,j,k)+field(2*ii,jj,kk))
     $                 + real*(-field(2*i-1,j,k)+field(2*ii-1,jj,kk))))
                  tre=0.5*(field(2*i-1,j,k)+field(2*ii-1,jj,kk)
     $                 - dir*(real*(field(2*i,j,k)+field(2*ii,jj,kk))
     $                 - imag*(-field(2*i-1,j,k)+field(2*ii-1,jj,kk))))
                  qua=0.5*(-field(2*i,j,k)+field(2*ii,jj,kk)
     $                 + dir*(imag*(field(2*i,j,k)+field(2*ii,jj,kk))
     $                 + real*(-field(2*i-1,j,k)+field(2*ii-1,jj,kk))))
                  field(2*i-1 ,j ,k)  = uno
                  field(2*i   ,j ,k)  = due
                  field(2*ii-1,jj,kk) = tre
                  field(2*ii  ,jj,kk) = qua
               endif
c               print *,1,i,j,k,ii,jj,kk,uno,due,tre,qua
c
            enddo
            temp = real
            real = real + real*real_inc - imag*imag_inc
            imag = imag + imag*real_inc + temp*imag_inc
         enddo
c
      enddo
c
c Bye!
c
      return
      end
c
c
c
      subroutine true_fft(ng,dir,field)
c
c Performs the true FFT for a complex 3D field
c
      implicit none
      integer ng,dir,ngtot
      real field(2,ng*ng*ng/2)
c
      real*8 pi8
      parameter (pi8=3.1415926535897932d0*2d0)
c
      integer i,j,k,l,m,n,ii,jj,kk,ll,mm,i1,j1,k1,dim
      real tmp,tmp_real,tmp_imag
      real*8 temp,teta,real,imag,real_inc,imag_inc
c
      ngtot=ng**3/2
      l=1
c
      do dim=1,3
c
c Here it does bit-reversal
c
         if (dim.eq.1) then
            n = ng/2
         else
            n = ng
         endif
         m = ngtot/(l*n)
         i1=l
         j1=i1*n
         k1=j1*m
         jj=1
         do j=1,j1,i1
            if (j.lt.jj) then
               do i=j,j+i1-1
                  do k=i,k1,j1
                     kk=jj+k-j
                     do ii=1,2
                        tmp=field(ii,k)
                        field(ii,k)=field(ii,kk)
                        field(ii,kk)=tmp
                     enddo
                  enddo
               enddo
            endif
            mm=j1/2
            do while (mm.ge.i1.and.jj.gt.mm)
               jj=jj-mm
               mm=mm/2
            enddo
            jj=jj+mm
         enddo
c
c Here it applies the Danielson-Lanczos lemma
c
         mm=i1
         do while (mm.lt.j1)
            ll=2*mm
            teta=dir*pi8*i1/ll
            real_inc=-2.d0*sin(.5d0*teta)**2
            imag_inc=sin(teta)
            real=1.d0
            imag=0.d0
c
            do k=1,mm,i1
               do j=k,k+i1-1
                  do i=j,k1,ll
                     kk=i+mm
                     tmp_real=real*field(1,kk)-imag*field(2,kk)
                     tmp_imag=real*field(2,kk)+imag*field(1,kk)
                     field(1,kk)=field(1,i)-tmp_real
                     field(2,kk)=field(2,i)-tmp_imag
                     field(1,i)=field(1,i)+tmp_real
                     field(2,i)=field(2,i)+tmp_imag
                  enddo
               enddo
c
c Recurrence
c
               temp=real
               real=real+real*real_inc-imag*imag_inc
               imag=imag+imag*real_inc+temp*imag_inc
            enddo
            mm=ll
         enddo
c
         l=l*n
      enddo
c
c Bye!
c
      return
      end
c
c
c
c DOUBLE PRECISION VERSIONS
c
c
c
      subroutine fft8(field,fnyq,ng,dir)
c
c FFT of a 3D field for a cubic grid
c NB can be easily generalized to non-cubic grids!
c
      implicit none
      integer ng,dir,i,j,k
      real*8 field(ng,ng,ng),fnyq(2,ng,ng)
c
      if (dir.eq.1) then
         call true_fft8(ng,dir,field)
         do k=1,ng
            do j=1,ng
               do i=1,2
                  fnyq(i,j,k)=field(i,j,k)
               enddo
            enddo
         enddo
         call disentangle8(ng,dir,field,fnyq)
      else
         call disentangle8(ng,dir,field,fnyq)
         call true_fft8(ng,dir,field)
      endif
c
c Bye!
c
      return
      end
c
c
c
      subroutine disentangle8(ng,dir,field,fnyq)
c
c Disentangles the Nyquist frequencies
c
      implicit none
      integer ng,dir
      real*8 field(ng,ng,ng),fnyq(2,ng,ng)
c
      real*8 pi8
      parameter (pi8=3.1415926535897932d0*2.d0)
      real*8 teta,real,imag,real_inc,imag_inc,temp
      integer i,j,k,ii,jj,kk
      real*8 uno,due,tre,qua
c
      teta=pi8*dir/dble(ng)
      real_inc=-2.0d0*sin(0.5d0*teta)**2
      imag_inc=sin(teta)
c
c Disentangles Nyquist frequencies
c

      do k=1,ng
         kk=ng-k+2
         if (k.eq.1) kk=1 
c
         real=1.0d0
         imag=0.0d0
c
         do i=1,ng/4+1
            ii=ng/2-i+2
            do j=1,ng
               jj=ng-j+2
               if (j.eq.1) jj=1
c
               if (i.eq.1) then
                  uno=0.5*(field(1,j,k)+fnyq(1,jj,kk)
     $                 +dir*(field(2,j,k)+fnyq(2,jj,kk)))
                  due=0.5*(field(2,j,k)-fnyq(2,jj,kk)
     $                 +dir*(-field(1,j,k)+fnyq(1,jj,kk)))
                  tre=0.5*(field(1,j,k)+fnyq(1,jj,kk)
     $                 -dir*(field(2,j,k)+fnyq(2,jj,kk)))
                  qua=0.5*(-field(2,j,k)+fnyq(2,jj,kk)
     $                 +dir*(-field(1,j,k)+fnyq(1,jj,kk)))
                  field(1,j,k)   = uno
                  field(2,j,k)   = due
                  fnyq (1,jj,kk) = tre
                  fnyq (2,jj,kk) = qua
               else
                  uno=0.5*(field(2*i-1,j,k)+field(2*ii-1,jj,kk)
     $                 + dir*(real*(field(2*i,j,k)+field(2*ii,jj,kk))
     $                 - imag*(-field(2*i-1,j,k)+field(2*ii-1,jj,kk))))
                  due=0.5*(field(2*i,j,k)-field(2*ii,jj,kk)
     $                 + dir*(imag*(field(2*i,j,k)+field(2*ii,jj,kk))
     $                 + real*(-field(2*i-1,j,k)+field(2*ii-1,jj,kk))))
                  tre=0.5*(field(2*i-1,j,k)+field(2*ii-1,jj,kk)
     $                 - dir*(real*(field(2*i,j,k)+field(2*ii,jj,kk))
     $                 - imag*(-field(2*i-1,j,k)+field(2*ii-1,jj,kk))))
                  qua=0.5*(-field(2*i,j,k)+field(2*ii,jj,kk)
     $                 + dir*(imag*(field(2*i,j,k)+field(2*ii,jj,kk))
     $                 + real*(-field(2*i-1,j,k)+field(2*ii-1,jj,kk))))
                  field(2*i-1 ,j ,k)  = uno
                  field(2*i   ,j ,k)  = due
                  field(2*ii-1,jj,kk) = tre
                  field(2*ii  ,jj,kk) = qua
               endif
c               print *,1,i,j,k,ii,jj,kk,uno,due,tre,qua
c
            enddo
            temp = real
            real = real + real*real_inc - imag*imag_inc
            imag = imag + imag*real_inc + temp*imag_inc
         enddo
c
      enddo
c
c Bye!
c
      return
      end
c
c
c
      subroutine true_fft8(ng,dir,field)
c
c Performs the true FFT for a complex 3D field
c
      implicit none
      integer ng,dir,ngtot
      real*8 field(2,ng*ng*ng/2)
c
      real*8 pi8
      parameter (pi8=3.1415926535897932d0*2d0)
c
      integer i,j,k,l,m,n,ii,jj,kk,ll,mm,i1,j1,k1,dim
      real*8 tmp,tmp_real,tmp_imag
      real*8 temp,teta,real,imag,real_inc,imag_inc
c
      ngtot=ng**3/2
      l=1
c
      do dim=1,3
c
c Here it does bit-reversal
c
         if (dim.eq.1) then
            n = ng/2
         else
            n = ng
         endif
         m = ngtot/(l*n)
         i1=l
         j1=i1*n
         k1=j1*m
         jj=1
         do j=1,j1,i1
            if (j.lt.jj) then
               do i=j,j+i1-1
                  do k=i,k1,j1
                     kk=jj+k-j
                     do ii=1,2
                        tmp=field(ii,k)
                        field(ii,k)=field(ii,kk)
                        field(ii,kk)=tmp
                     enddo
                  enddo
               enddo
            endif
            mm=j1/2
            do while (mm.ge.i1.and.jj.gt.mm)
               jj=jj-mm
               mm=mm/2
            enddo
            jj=jj+mm
         enddo
c
c Here it applies the Danielson-Lanczos lemma
c
         mm=i1
         do while (mm.lt.j1)
            ll=2*mm
            teta=dir*pi8*i1/ll
            real_inc=-2.d0*sin(.5d0*teta)**2
            imag_inc=sin(teta)
            real=1.d0
            imag=0.d0
c
            do k=1,mm,i1
               do j=k,k+i1-1
                  do i=j,k1,ll
                     kk=i+mm
                     tmp_real=real*field(1,kk)-imag*field(2,kk)
                     tmp_imag=real*field(2,kk)+imag*field(1,kk)
                     field(1,kk)=field(1,i)-tmp_real
                     field(2,kk)=field(2,i)-tmp_imag
                     field(1,i)=field(1,i)+tmp_real
                     field(2,i)=field(2,i)+tmp_imag
                  enddo
               enddo
c
c Recurrence
c
               temp=real
               real=real+real*real_inc-imag*imag_inc
               imag=imag+imag*real_inc+temp*imag_inc
            enddo
            mm=ll
         enddo
c
         l=l*n
      enddo
c
c Bye!
c
      return
      end
